
# q-expansion of newform 65.6.a.d, downloaded from the LMFDB on 13 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 65
weight = 6
poly_data = [-19440, 6624, 6120, -8, -163, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0], 1], [[-12312, -30420, -1814, 851, 18, -5], 2016], [[-96552, 30420, 3830, -851, -18, 5], 2016], [[29880, 3060, -3098, -103, 30, 1], 336], [[80856, -41148, -1994, 2705, -18, -23], 2016]]

hecke_ring_character_values = None
aps_data = [[0, 1, 0, 0, 0, 0], [6, 1, 1, 0, 0, 0], [-25, 0, 0, 0, 0, 0], [38, -6, -1, 2, 0, 3], [-24, 13, -7, -4, -1, 6], [169, 0, 0, 0, 0, 0], [130, -66, -27, 2, 1, -1], [192, -77, 23, -20, 3, -10], [1450, -109, 102, 2, -1, -25], [1340, -258, 172, -28, 36, 10], [448, 389, 52, 14, -18, -35], [2360, 696, -169, -22, 1, -9], [494, 90, -237, 334, 11, 121], [614, -903, 69, -48, 82, 0], [-1742, -290, -331, -26, 144, -127], [1178, -1714, -259, -30, 53, -17], [-4552, -1055, -489, 112, -165, 292], [-140, -2026, 1030, -160, 74, 124], [10510, 668, -1419, 34, -430, 91], [-11940, 1727, -1312, 290, 394, -45], [8776, -5236, -1817, -214, -169, 31], [36660, 3730, -3378, 344, -104, -652], [21014, 1682, -1291, 678, 390, 121], [15742, 8924, -486, -460, 338, -274], [41394, 2176, 0, -448, 672, 1040], [-29862, 3292, 3394, -524, -1014, 822], [87442, 4889, -424, 138, 447, 715], [10634, -1563, -665, 1484, -1234, 226], [-12790, -2928, 1310, -1276, -942, 990], [-31570, -5426, 2221, 290, -1055, -665], [19862, -3321, -2138, -2774, 1539, 3], [-84408, -3492, 370, -3044, 1032, -1590], [-34794, 6500, -1622, 4276, 330, 390], [568, -16282, 3672, -1604, -202, -2798], [-135906, 9488, 8028, 2280, 256, -1644], [212596, -5947, 3540, -1970, -2010, -419], [68078, 9120, -758, -4148, -2706, 2250], [47842, -6700, -6399, -686, 2328, 1051], [-23474, 9782, 1017, 3398, -630, -4063], [-61986, -1378, -18435, -1950, -4207, -537], [9120, -28112, -6748, 7376, -2274, -4080], [264804, -254, -6578, 6312, 966, -2824], [-87624, 4484, -21104, -11864, 948, -4500], [127302, -17144, 6736, 3648, 2348, -6448], [-136530, 20956, 26450, -4876, 4122, -1050], [256900, 13940, -19040, -2560, -90, 1000], [-88376, 26204, 1288, 3536, -3286, -320], [502486, 19034, 19523, 9066, -8004, -1817], [-613838, 21144, 29571, -2650, -3086, 2393], [-149194, 83886, -13895, -5030, 7817, -2181], [-83758, -36904, -4818, -956, 1278, 958], [-445980, -69969, 1250, 19478, 2922, 1953], [-75434, -3790, -43499, 218, 2617, 6307], [160492, -105558, -22456, -7028, 920, 3458], [-281402, 58544, -43930, 8884, 5798, -2098], [138114, -109001, 23496, 5742, 4547, 15393], [-35406, -56296, 24568, 14032, -8080, 8592], [150812, -9927, -47208, 558, 5842, -12459], [182918, -43134, -43609, -3034, 4767, -18699], [129198, 8094, -61473, -26562, -6421, 6033], [860230, 104261, 46903, -10972, 4324, 5446], [-120268, 37396, 23695, -8110, -11783, -7541], [325290, 59962, -39713, -40038, 6958, -2569], [560676, 12306, -34220, -22196, -8134, 7674], [650810, 132210, -4491, -9798, 5225, -4365], [-1184260, -61408, -116119, -20490, 6515, -2951], [330424, -62245, -81063, 11104, 14239, -29468], [-328762, -28686, -57797, 25534, -15941, -3687], [1509106, 153147, 90549, 1868, -2266, 17546], [-552934, -39486, -38971, 56954, -20359, 29859], [1063716, -83492, 69065, -34666, 5901, -31983], [563560, -17, -66464, 28218, -16100, 1151], [1556726, -96411, 114546, 7158, -2641, -18579], [1406574, 38272, 146466, 31100, 16230, 21794], [-733204, 23913, -1453, 2888, 3271, -984], [573398, -38066, -80767, -8586, -30302, -575], [-2487482, 14032, 67536, -75760, 15000, 15680], [673336, -188668, 116337, 31630, -3629, 32437], [174710, -143340, -119818, 66300, -3894, 19834], [-43794, 455374, 18545, 1306, 21449, -3517], [-220836, -31058, 3600, -51420, 29972, -28026], [368786, 247884, 253750, 6172, -2446, 12834], [-839664, -280547, -30122, 62170, -9264, 4383], [-499014, 218044, -153120, 11672, -27972, 24260], [123896, 458256, -160714, -7372, 5178, 36726], [873014, -282799, 58695, -50108, -1094, -42458], [-2333510, -297882, 90241, 7866, -21391, 5723], [-660166, -308412, 7074, -28716, -6338, 10374], [1555794, 479270, -129183, -5334, 19305, 33387], [-1049810, -565120, -24763, 30330, 4870, -7817], [40578, 198825, 79197, -12144, 50986, -13032], [-936600, 336175, -44952, -38502, 11480, -77505], [1513958, 221688, 151497, -59934, -34650, 10923], [-1068316, -50730, 43114, 49040, -16034, 96016], [978912, -146105, 67573, 7368, 24353, -36160], [811938, 161717, 50630, -105490, 59139, -54447], [88090, 70994, 240865, 105234, 21729, 61319], [1148256, -94542, 356674, -96008, 4744, -114576], [3593414, 426477, -200097, 58956, -16698, -29166], [-1580818, -390654, 166501, -190070, -4535, -44733], [540570, 259189, 128561, 148888, -33920, 788], [-5054968, 66480, -76477, -25086, -18107, -34373], [-914090, 364257, -347027, -29760, -8622, 23648], [-2330572, 357354, -43610, 17744, -11644, -142532], [-5239728, -39842, 164144, 9868, -5214, 7514], [3050168, -225732, -102333, -12942, -34829, -52197], [-2439486, -160076, 472713, 84450, 25436, -18261], [-13350, -650604, 39334, 124780, -18398, 80010], [-5641164, -491274, 86288, 128588, 21102, 95226], [5238250, -681208, -82592, -22336, 54816, 27064], [4862670, 374143, 554168, -58226, -43281, -83983], [7271210, -206088, 124800, -40416, -34716, 41808], [5373398, 121000, 218140, -58648, 83160, 29452], [2558928, 273241, -52225, 57952, 79411, -46780], [2742664, 559139, -219126, -132842, -37408, 40705], [231362, -254876, 167418, -57212, -119638, 15670], [5069878, -340186, 356067, -15854, 25572, -147533], [3357054, -395609, 79240, 125494, 48493, 79773], [6995338, -1313056, -288354, -136540, 25142, -13306], [-7695556, -716150, 145684, -91852, -32452, -69170], [8414794, -1037812, -256698, 64684, -41162, 14314], [-595342, 954738, -546125, -45506, -17101, 123729], [-3265942, 94310, -55671, -193814, 71885, -94133], [-6347602, 136540, 219901, 52082, 9646, -22085], [-5241632, 1007, 207449, -55608, -54761, 17224], [-9846102, 500352, -169008, 11856, 27552, 74400], [8181790, -513802, 646557, -12494, -37519, -109649], [1170980, -133316, 310890, 123484, -59116, 124090], [760062, 1258645, -130670, 288078, -3391, -3799], [-4184938, -762456, 2992, 35680, -105860, -45888], [-1244164, -406053, 119089, -170696, -915, -60240], [2691046, -635662, -274595, -129018, 104490, -182599], [-3440728, 142650, -125546, -298208, -11666, 35208], [-4390466, 821258, -737481, -187322, 108939, -265163], [-620446, -91568, 17420, -68520, -38976, 146308], [-1188390, -1433828, -415752, -106072, 58836, 105308], [3938040, -1515796, -411131, -19370, -72929, 35601], [-1660846, 485022, -32021, -467894, 126310, -124161], [1237950, 370676, 277634, 11156, -237594, 19446], [-2456628, -125294, 140160, 257628, 6910, -157578], [-1242328, -1736289, 108719, 79292, 82157, 228390], [-3376162, 842868, 219944, -28440, 101876, 9124], [20659250, -1614699, 70910, 188870, -174491, 184581], [-2097314, 1636450, -391295, 66270, 19106, 193925], [12338332, -1753126, -217792, -467196, 117248, -162614], [6221516, -987455, 875644, 165614, -103836, -204251], [-3129076, 2777928, 229345, 109606, -70201, -178431], [-922186, 2417864, -14358, 132172, -14250, -24254], [13152052, -760126, -215498, 112976, -18658, 219184], [-8441898, 736230, 619675, 136066, -35988, 245499], [-5976686, -476416, -561264, -257216, -120496, -239216], [3151008, 1537894, -655334, 228544, 139560, -78660], [-6174306, 520585, -754203, -191584, 153554, -72592], [8709358, 166469, 172972, -165918, 12621, -23641], [-4913438, -1201783, 248365, -143200, -61954, 40192], [-1825284, -74484, -935644, -514792, 23910, -349284], [15926028, -2089646, -1444580, 249132, -48010, 243674], [8600082, 458462, -120463, 34010, -106267, -227589], [7043546, -1468908, 728336, 427544, -63104, 343716], [6799870, -950054, -1682193, -18682, -96977, 185909], [-7650090, 1635910, -388851, 27054, -38078, -39747], [6570718, 484740, 44560, -564072, 174812, -204196], [484230, -530456, 1271623, -20826, 43592, -114295], [-28586720, -2091048, 1249716, -450688, -30074, 113528], [3565704, -60692, 1126327, -102614, -77921, -29601], [-699778, 1318256, -624815, -172990, -150558, 176035], [-2369632, -647568, 651486, 546180, 96962, 45150], [10907538, 2027242, -814117, 107470, 156151, -273511]]
